@@ -7,7 +7,7 @@ use 5.014;
 
 use Scalar::Util qw( looks_like_number );
 
-use JSON::MaybeXS   qw( decode_json );
+use JSON::MaybeXS qw( decode_json );
 
 use Term::Choose            qw();
 use Term::Choose::Constants qw( WIDTH_CURSOR );
@@ -70,7 +70,6 @@ sub get_stmt {
     elsif ( $stmt_type eq 'Create_table' ) {
         my $stmt = sprintf "CREATE TABLE $qt_table (%s)", join ', ', map { $_ // '' } @{$sql->{create_table_cols}};
         @tmp = ( $sf->__stmt_fold( $stmt, $term_w, $indent0 ) );
-
     }
     elsif ( $stmt_type eq 'Create_view' ) {
         @tmp = ( $sf->__stmt_fold( "CREATE VIEW $qt_table", $term_w, $indent0 ) );
@@ -84,12 +83,12 @@ sub get_stmt {
         push @tmp, $sf->__stmt_fold( $sql->{having_stmt},   $term_w, $indent2, $sql->{having_args} ) if $sql->{having_stmt};
         push @tmp, $sf->__stmt_fold( $sql->{order_by_stmt}, $term_w, $indent2                      ) if $sql->{order_by_stmt};
         if ( $sf->{i}{driver} =~ /^(?:Firebird|DB2|Oracle)\z/ ) {
-            push @tmp, $sf->__stmt_fold( $sql->{offset_stmt},   $term_w, $indent2 ) if $sql->{offset_stmt};
-            push @tmp, $sf->__stmt_fold( $sql->{limit_stmt},    $term_w, $indent2 ) if $sql->{limit_stmt};
+            push @tmp, $sf->__stmt_fold( $sql->{offset_stmt}, $term_w, $indent2 ) if $sql->{offset_stmt};
+            push @tmp, $sf->__stmt_fold( $sql->{limit_stmt},  $term_w, $indent2 ) if $sql->{limit_stmt};
         }
         else {
-            push @tmp, $sf->__stmt_fold( $sql->{limit_stmt},    $term_w, $indent2 ) if $sql->{limit_stmt};
-            push @tmp, $sf->__stmt_fold( $sql->{offset_stmt},   $term_w, $indent2 ) if $sql->{offset_stmt};
+            push @tmp, $sf->__stmt_fold( $sql->{limit_stmt},  $term_w, $indent2 ) if $sql->{limit_stmt};
+            push @tmp, $sf->__stmt_fold( $sql->{offset_stmt}, $term_w, $indent2 ) if $sql->{offset_stmt};
         }
     }
     elsif ( $stmt_type eq 'Delete' ) {
@@ -279,26 +278,22 @@ sub stmt_placeholder_to_value {
 
 sub alias {
     my ( $sf, $sql, $type, $identifier, $default ) = @_;
-    my $term_w = get_term_width();
-    my $tmp_info;
-    if ( $identifier eq '' ) { # Union
-        $identifier .= 'UNION Alias: ';
-    }
-    elsif ( print_columns( $identifier . ' AS ' ) > $term_w / 3 ) {
-        $tmp_info = 'Alias: ' . "\n" . $identifier;
-        $identifier = 'AS ';
-    }
-    else {
-        $tmp_info = 'Alias: ';
-        $identifier .= ' AS ';
+    if ( defined $default ) { # && ! $sf->{o}{G}{quote_identifiers} ) {
+        if ( $sf->{i}{driver} eq 'Pg' ) {
+            $default = lc $default;
+        }
+        elsif ( $sf->{i}{driver} =~ /^(?:Firebird|DB2)\z/ ) {
+            $default = uc $default;
+        }
     }
+    my $prompt = 'AS ';
     my $alias;
     if ( $sf->{o}{alias}{$type} ) {
         my $tr = Term::Form::ReadLine->new( $sf->{i}{tr_default} );
-        my $info = $sf->get_sql_info( $sql ) . "\n" . $tmp_info;
+        my $info = $sf->get_sql_info( $sql ) . "\n" . $identifier;
         # Readline
         $alias = $tr->readline(
-            $identifier,
+            $prompt,
             { info => $info }
         );
         $sf->print_sql_info( $info );
@@ -306,9 +301,6 @@ sub alias {
     if ( ! length $alias ) {
         $alias = $default;
     }
-    if ( $sf->{i}{driver} eq 'Pg' && ! $sf->{o}{G}{quote_identifiers} ) {
-        return lc $alias;
-    }
     return $alias;
 }
 
@@ -352,16 +344,24 @@ sub quote_col_qualified {
 
 
 sub quote_cols {
-    my ( $sf, $list ) = @_;
+    my ( $sf, $cols ) = @_;
     if ( $sf->{o}{G}{quote_identifiers} ) {
-        return [ map { $sf->{d}{dbh}->quote_identifier( $_ ) } @$list ];
+        return [ map { $sf->{d}{dbh}->quote_identifier( $_ ) } @$cols ];
     }
     else {
-        return [ @$list ];
+        return [ @$cols ];
     }
 }
 
 
+sub unquote_identifier {
+    my ( $sf, $identifier ) = @_;
+    my $qc = quotemeta( $sf->{d}{identifier_quote_char} );
+    $identifier =~ s/$qc(?=(?:$qc$qc)*(?:[^$qc]|\z))//g;
+    return $identifier;
+}
+
+
 sub backup_href {
     my ( $sf, $href ) = @_;
     my $backup = {};
@@ -429,9 +429,10 @@ sub sql_limit {
 }
 
 
-sub tables_column_names_and_types { # db
+sub tables_column_names_and_types {
     my ( $sf, $table_keys ) = @_;
     my ( $col_names, $col_types );
+    # without `LIMIT 0` slower with big tables: mysql, MariaDB and Pg
     for my $table_k ( @$table_keys ) {
         if ( ! eval {
             my $sth = $sf->{d}{dbh}->prepare( "SELECT * FROM " . $sf->quote_table( $sf->{d}{tables_info}{$table_k} ) . $sf->sql_limit( 0 ) );
@@ -449,6 +450,8 @@ sub tables_column_names_and_types { # db
 
 sub column_names {
     my ( $sf, $qt_table ) = @_;
+    # without `LIMIT 0` slower with big tables: mysql, MariaDB and Pg
+    # no difference with SQLite, Firebird, DB2 and Informix
     my $sth = $sf->{d}{dbh}->prepare( "SELECT * FROM " . $qt_table . $sf->sql_limit( 0 ) );
     $sth->execute() if $sf->{i}{driver} ne 'SQLite';
     return [ @{$sth->{NAME}} ];
