@@ -5,7 +5,7 @@ use warnings;
 use strict;
 use 5.014;
 
-our $VERSION = '2.314';
+our $VERSION = '2.315';
 
 #use bytes; # required
 use Scalar::Util qw( looks_like_number );
@@ -69,21 +69,24 @@ sub get_db_handle {
 
 
 sub get_schemas {
-    my ( $sf, $dbh, $db ) = @_;
-    my ( $user_schema, $sys_schema );
+    my ( $sf, $dbh, $db, $is_system_db ) = @_;
+    my ( $user_schemas, $sys_schemas );
     if ( $sf->{Plugin}->can( 'get_schemas' ) ) {
-        ( $user_schema, $sys_schema ) = $sf->{Plugin}->get_schemas( $dbh, $db );
+        ( $user_schemas, $sys_schemas ) = $sf->{Plugin}->get_schemas( $dbh, $db, $is_system_db ); ##
     }
     else {
         my $driver = $dbh->{Driver}{Name}; #
         if ( $driver eq 'SQLite' ) {
-            $user_schema = [ 'main' ]; # [ undef ];
+            $user_schemas = [ 'main' ]; # [ undef ];
         }
         elsif( $driver =~ /^(?:mysql|MariaDB)\z/ ) {
             # MySQL 8.0 Reference Manual / MySQL Glossary / Schema:
             # In MySQL, physically, a schema is synonymous with a database.
             # You can substitute the keyword SCHEMA instead of DATABASE in MySQL SQL syntax,
-            $user_schema = [ $db ];
+            $user_schemas = [ $db ];
+        }
+        elsif ( $driver eq 'Informix' ) {
+            ( $user_schemas, $sys_schemas ) = $sf->__informix_schemas( $dbh );
         }
         else {
             my $table_schem;
@@ -92,7 +95,7 @@ sub get_schemas {
                 $table_schem = 'pg_schema';
             }
             elsif ( $driver eq 'Sybase' ) {
-                # DBD::Sybase  table_info
+                # table_info
                 $table_schem = 'TABLE_OWNER';
             }
             else {
@@ -109,17 +112,20 @@ sub get_schemas {
             my $info = $sth->fetchall_hashref( $table_schem );
             for my $schema ( sort keys %$info ) {
                 if ( defined $regex_sys && $schema =~ $regex_sys ) {
-                    push @$sys_schema, $schema;
+                    push @$sys_schemas, $schema;
                 }
                 else {
-                    push @$user_schema, $schema;
+                    push @$user_schemas, $schema;
                 }
             }
         }
     }
-    $user_schema = [] if ! defined $user_schema;
-    $sys_schema  = [] if ! defined $sys_schema;
-    return $user_schema, $sys_schema;
+    $user_schemas //= [];
+    $sys_schemas //= [];
+    if ( $is_system_db ) {
+        return [], [ @$user_schemas, @$sys_schemas ];
+    }
+    return $user_schemas, $sys_schemas;
 }
 
 
@@ -171,6 +177,9 @@ sub get_databases {
 sub tables_info { # not public
     my ( $sf, $dbh, $schema, $is_system_schema, $has_attached_db ) = @_;
     my $driver = $sf->get_db_driver();
+    if ( $driver eq 'Informix' ) {
+        return $sf->__informix_tables_info( $dbh, $schema, $is_system_schema );
+    }
     my ( $table_cat, $table_schem, $table_name );
     if ( $driver eq 'Pg' ) {
         $table_cat   = 'TABLE_CAT';
@@ -184,7 +193,7 @@ sub tables_info { # not public
         # ...
     }
     elsif ( $driver eq 'Sybase' ) {
-        # DBD::Sybase  table_info
+        # table_info
         $table_cat   = 'TABLE_QUALIFIER';
         $table_schem = 'TABLE_OWNER';
         $table_name  = 'TABLE_NAME';
@@ -213,40 +222,88 @@ sub tables_info { # not public
         if ( $info_table->{TABLE_TYPE} eq 'INDEX' ) {
             next;
         }
-        # The table name in $table_keys is used in the tables-menu but not in SQL code.
+        # The table name in $table_key is used in the tables-menu but not in SQL code.
         # To get the table names for SQL code it is used the 'quote_table' routine in Auxil.pm.
-        my $table_keys;
+        my $table_key;
         if ( $driver eq 'SQLite' && ! defined $schema ) {
             # The $schema is `undef` if a SQLite database has attached databases.
-            next if $info_table->{$table_name} eq 'sqlite_temp_master'; # no 'create temp table'
+            next if $info_table->{$table_name} eq 'sqlite_temp_master'; # 'create temp table' not supported
             if ( $info_table->{$table_schem} =~ /^main\z/i ) {
-                $table_keys = sprintf "[%s] %s", "\x{001f}" . $info_table->{$table_schem}, $info_table->{$table_name};
+                $table_key = sprintf "[%s] %s", "\x{001f}" . $info_table->{$table_schem}, $info_table->{$table_name};
                 # \x{001f} keeps the main tables on top of the tables menu.
             }
             else {
-                $table_keys = sprintf "[%s] %s", $info_table->{$table_schem}, $info_table->{$table_name};
+                $table_key = sprintf "[%s] %s", $info_table->{$table_schem}, $info_table->{$table_name};
             }
         }
         else {
-            $table_keys = $info_table->{$table_name};
+            $table_key = $info_table->{$table_name};
         }
         if ( $is_system_schema ) {
-            push @sys_table_keys, $table_keys;
+            push @sys_table_keys, $table_key;
         }
         else {
             if ( $info_table->{TABLE_TYPE} =~ /^SYSTEM/ || ( $driver eq 'SQLite' && $info_table->{TABLE_NAME} =~ /^sqlite_/ ) ) {
-                push @sys_table_keys, $table_keys;
+                push @sys_table_keys, $table_key;
             }
             else {
-                push @user_table_keys, $table_keys;
+                push @user_table_keys, $table_key;
             }
         }
-        $tables_info->{$table_keys} = [ @{$info_table}{@keys} ];
+        $tables_info->{$table_key} = [ @{$info_table}{@keys} ];
     }
     return $tables_info, [ sort @user_table_keys ], [ sort @sys_table_keys ];
 }
 
 
+sub __informix_schemas {
+    my ( $sf, $dbh ) = @_;
+    my $stmt = qq{SELECT owner FROM informix.systables GROUP BY owner};
+    my $owners = $dbh->selectcol_arrayref( $stmt, {} );
+    my @user_schemas;
+    my @sys_schemas;
+    for ( @$owners ) {
+        if ( ! /^(?:\p{L}|_)/ ) {
+            next;
+        }
+        elsif ( /^informix/i ) {
+            push @sys_schemas, $_;
+        }
+        else {
+            push @user_schemas, $_;
+        }
+    }
+    return [ sort @user_schemas ], [ sort @sys_schemas ];
+}
+
+
+sub __informix_tables_info {
+    my ( $sf, $dbh, $schema, $is_system_schema ) = @_;
+    my $stmt = qq{SELECT owner, tabname, tabtype FROM informix.systables WHERE owner = ? ORDER BY tabname};
+    my $raw_info = $dbh->selectall_arrayref( $stmt, { Slice => {} }, ( $schema ) );
+    my %map_table_type = (
+        T => 'TABLE',       E => 'EXTERNAL TABLE',      V => 'VIEW',
+        Q => 'SEQUENCE',    P => 'PRIVATE SYNONYM',     S => 'PUBLIC SYNONYM',
+    );
+    my $tables_info = {};
+    my $user_table_keys = [];
+    my $sys_table_keys = [];
+    for ( @$raw_info ) {
+        #next if $_->{tabtype} eq 'Q';
+        $tables_info->{$_->{tabname}} = [ undef, $_->{owner}, $_->{tabname}, $map_table_type{$_->{tabtype}} ];
+        if ( $is_system_schema ) {
+            push @$sys_table_keys, $_->{tabname};
+        }
+        else {
+            push @$user_table_keys, $_->{tabname};
+        }
+    }
+    return $tables_info, $user_table_keys, $sys_table_keys;
+}
+
+
+
+
 
 # Oracle and Sybase untested
 
@@ -254,6 +311,8 @@ sub tables_info { # not public
 #   SET quoted_identifier ON
 
 
+
+
 1;
 
 
@@ -269,7 +328,7 @@ App::DBBrowser::DB - Database plugin documentation.
 
 =head1 VERSION
 
-Version 2.314
+Version 2.315
 
 =head1 DESCRIPTION
 
